# -*- coding: utf-8 -*-
"""Israeli_Palestinian_CodeFINAL.ipynb

Automatically generated by Colab.

Original file is located at
    https://colab.research.google.com/drive/1l_bIEAs4ezknhqL77h_Jx-DcCvPiAEu0
"""

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import plotly as ply
from scipy import stats
import statsmodels.api as sm
from statsmodels.formula.api import ols
from statsmodels.stats.multicomp import pairwise_tukeyhsd
from statsmodels.multivariate.manova import MANOVA
import os
from itertools import combinations
from sklearn.preprocessing import StandardScaler
from statsmodels.miscmodels.ordinal_model import OrderedModel

# If you are running in Colab
#from google.colab import drive
#drive.mount('/content/drive')

# Change folders to access the mounted drive and data file location
#os.chdir('PATH HERE')

# # Read in the csv file and print out the column names
#data_raw = pd.read_csv('ces_public_2023.csv')
# data_raw.columns

# Columns for discussion reluctance
columns_discuss = ['israelipalestinian_discuss', 'politics_discuss', 'trans_discuss',
 'abortion_discuss', 'religion_discuss', 'genderroles_discuss', 'sexualorientation_discuss',
 'race_discuss', 'covid_discuss', 'freespeech_discuss', 'typical_discuss']

"""**Figure 1 & Analysis**"""

# Recode responses into numerical values
resp_coding = {"Very comfortable": 0,
               "Somewhat comfortable": 0,
               "Somewhat reluctant": 1,
               "Very reluctant": 2}

# Apply recoding
data_recoded = data_raw[columns_discuss].replace(resp_coding)

# Calculate the proportions
proportions = {}
for topic in columns_discuss:
    counts = data_recoded[topic].value_counts(normalize=True)
    proportions[topic] = {
        "Somewhat reluctant": counts.get(1, 0),
        "Very reluctant": counts.get(2, 0)
    }

print(proportions)

# Convert to DataFrame
df_proportions = pd.DataFrame(proportions).transpose()

# Plotting the stacked bar chart
fig, ax = plt.subplots(figsize=(14, 8))

# Define the colors
colors = ['salmon', 'lightblue']

# Plot each category in a stacked bar
bottom = None
for i, column in enumerate(df_proportions.columns):
    ax.bar(df_proportions.index, df_proportions[column], bottom=bottom, color=colors[i], label=column)
    if bottom is None:
        bottom = df_proportions[column]
    else:
        bottom += df_proportions[column]

# Set labels and title
ax.set_xlabel('Topic')
ax.set_ylabel('Proportion')
ax.set_title('Proportion of Reluctant Students by Topic')

# Rotate x-tick labels for better readability
ax.set_xticklabels(df_proportions.index, rotation=45, ha='right')

# Add legend
ax.legend(title='Reluctance Level')

# Show Figure 1 of report
plt.tight_layout()
plt.show()

"""**Figure 2 & Analysis**"""

import pandas as pd
import matplotlib.pyplot as plt
from scipy import stats

# Insert path here to folder holding the data
path = 'PATH HERE'

# # Read in the data
data = pd.read_csv(path+'data_public.csv')

# Political party variable name
political_party = "politicalparty_demo"

# Recode responses into numerical values
resp_coding = {"Very comfortable": 0,
               "Somewhat comfortable": 1,
               "Somewhat reluctant": 2,
               "Very reluctant": 3}

# Create a copy of the data and apply recoding
data_recoded = data.copy()
data_recoded['israelipalestinian_discuss'] = data_recoded['israelipalestinian_discuss'].replace(resp_coding)

# Print the unique values of the political_party column
print(data_recoded[political_party].unique())

# Create a contingency table
contingency_table = pd.crosstab(data_recoded[political_party], data_recoded['israelipalestinian_discuss'])

# Perform the Chi-Square Test of Independence
chi2, p, df, expected = stats.chi2_contingency(contingency_table)

# Print the results
print("Chi-Square Test Statistic:", chi2)
print("p-value:", p)
print("Degrees of Freedom:", df)
print("Expected Frequencies:")
print(expected)

print(contingency_table)

# Calculate the proportions for each category
proportions = contingency_table.div(contingency_table.sum(axis=1), axis=0)

# Calculate the total reluctance (sum of "Somewhat reluctant" and "Very reluctant")
proportions['total_reluctant'] = proportions[2] + proportions[3]

# Sort the proportions by total reluctance in descending order
proportions = proportions.sort_values(by='total_reluctant', ascending=True)

# Drop the total_reluctant column as it's no longer needed
proportions = proportions.drop(columns=['total_reluctant'])

# Define the response labels
response_labels = ["Very comfortable", "Somewhat comfortable", "Somewhat reluctant", "Very reluctant"]

# Define custom colors: Green, Yellow, Orange, Red (using the previous colors)
colors = ['#01665e', '#5ab4ac', '#d8b365', '#8c510a']

# Create the diverging stacked bar chart
fig, ax = plt.subplots(figsize=(12, 8))

# Plot the bars
ax.barh(proportions.index, -proportions[3], color=colors[3], edgecolor='none', label="Very reluctant")
ax.barh(proportions.index, -proportions[2], left=-proportions[3], color=colors[2], edgecolor='none', label="Somewhat reluctant")
ax.barh(proportions.index, proportions[1], color=colors[1], edgecolor='none', label="Somewhat comfortable")
ax.barh(proportions.index, proportions[0], left=proportions[1], color=colors[0], edgecolor='none', label="Very comfortable")

# Add the legend
ax.legend(title="Response", bbox_to_anchor=(1.05, 1), loc='upper left')

# Add the grid
ax.grid(True, axis='x', linestyle='--', linewidth=0.7, color='grey', alpha=0.6)

# Set labels and title
ax.set_xlabel("Proportion of Responses")
ax.set_ylabel("Political Party")
# ax.set_title("Proportion of Responses for Israelipalestinian Discussion by Political Party")

# Set x-axis limits
ax.set_xlim(-1, 1)

# Customize x-axis tick labels
ax.set_xticklabels([str(abs(x)) for x in ax.get_xticks()])

# Center the bars at 0
ax.axvline(0, color='black', linestyle='-', linewidth=1)

plt.tight_layout()

# Save the plot as a PNG file
plt.savefig('diverging_stacked_bar_chart_israelipalestinian.png', dpi=300)

# Show the Figure 2 plot
plt.show()

"""**Figure 3 & Analysis**"""

import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.preprocessing import StandardScaler
from statsmodels.miscmodels.ordinal_model import OrderedModel

path = 'PATH HERE'

# Read in the data
data = pd.read_csv(path+'data_public.csv')

# Discussion topic column names
columns_discuss = ['israelipalestinian_discuss']

# Controls
controls = ['politics_overall', 'politics_social', 'gender_recoded', 'extraversion', 'neuroticism', 'agreeableness',
            'conscientiousness', 'openness', 'typical_discuss']

# Dictionary to recode the political spectrum to be binary Left (0) vs Right (1)
politics_coding = {"Very right": 5,
                   "Right": 4,
                   "Lean right": 3,
                   "Lean left": 2,
                   "Left": 1,
                   "Very left": 0}

# Dictionary to recode the reluctance ratings
resp_coding = {"Very comfortable": 1,
               "Somewhat comfortable": 2,
               "Somewhat reluctant": 3,
               "Very reluctant": 4}

# Dictionary to recode gender
gender_coding = {"Male": 0,
                 "Other": 1,
                 "Female": 2}

# Recode the data
data['politics_economic'] = data['politics_economic'].replace(politics_coding)
data['politics_social'] = data['politics_social'].replace(politics_coding)
data['politics_overall'] = data['politics_overall'].replace(politics_coding)
data['gender_recoded'] = data['gender_recoded'].replace(gender_coding)
data[columns_discuss] = data[columns_discuss].replace(resp_coding)
data['typical_discuss'] = data['typical_discuss'].replace(resp_coding)

# Drop NaN from the political spectrum question
columns_interest = ['israelipalestinian_discuss', 'politics_discuss', 'trans_discuss',
 'abortion_discuss', 'religion_discuss', 'genderroles_discuss', 'sexualorientation_discuss',
 'race_discuss', 'covid_discuss', 'freespeech_discuss', 'typical_discuss', 'politics_overall','gender_recoded',
            'extraversion', 'neuroticism', 'agreeableness', 'conscientiousness', 'openness']

for col in columns_interest:
  data.dropna(subset=[col], inplace=True)

# Create quadratic term for politics_overall
data['politics_overall_squared'] = data['politics_overall'] ** 2

# Update controls to include the quadratic term
controls = ['politics_overall', 'politics_overall_squared', 'politics_social',
            'gender_recoded', 'extraversion', 'neuroticism', 'agreeableness',
            'conscientiousness', 'openness', 'typical_discuss']

# Calculate the average reluctance score for the Israeli-Palestinian variable
average_reluctance_israelipalestinian = data.groupby('politics_overall')['israelipalestinian_discuss'].mean().reset_index()

# Plot the average reluctance score
plt.figure(figsize=(10, 6))
sns.barplot(x='politics_overall', y='israelipalestinian_discuss', data=average_reluctance_israelipalestinian, palette='viridis')
plt.xlabel('Political Orientation', fontsize=12, fontweight='bold')
plt.ylabel('Overall Reluctance', fontsize=12, fontweight='bold')
plt.title('Average Reluctance Score for Israeli-Palestinian Discussions by Political Orientation', fontsize=14, fontweight='bold')
plt.xticks(ticks=range(6), labels=["Very Left", "Left", "Lean Left", "Lean Right", "Right", "Very Right"], fontsize=10, fontweight='bold')
plt.yticks(fontsize=10, fontweight='bold')
plt.ylim(1, 4)
plt.tight_layout()
plt.show()

# Standardize (convert to z-scores) using standard scaler
scaler = StandardScaler()
data[columns_discuss + controls] = scaler.fit_transform(data[columns_discuss + controls])

def fit_ordinal_model(data, predictors, outcome):
    model = OrderedModel(data[outcome], data[predictors], distr='logit')
    result = model.fit(method='bfgs', disp=False)
    print(f"Results for {outcome}:")
    print(result.summary())

    # Extract standardized coefficients (effect sizes)
    std_coefficients = result.params
    print("Standardized Coefficients (Effect Sizes):")
    print(std_coefficients)

    return result

results = {}

for col in columns_discuss:
    print(col)
    results[col] = fit_ordinal_model(data, controls, col)

"""**Figure 4 & Analysis**"""

import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.preprocessing import StandardScaler
from statsmodels.miscmodels.ordinal_model import OrderedModel

# Insert path here to folder holding the data
path = 'PATH HERE'

# Read in the data
data = pd.read_csv(path + 'data_public.csv')

# Convert 'freeexpression_campu' to an ordered categorical type
data['prevented_offensive'] = pd.Categorical(
    data['freeexpression_campu'],
    categories=["Strongly disagree", "Somewhat disagree", "Somewhat agree", "Strongly agree"],
    ordered=True
)

views_encouraged_code = {'Never': 0,
                         'Very rarely': 1,
                         'Rarely': 2,
                         'Occasionally': 3,
                         'Frequently': 4,
                         'Very frequently': 5}

data['views_encouraged'] = data['views_encouraged'].map(views_encouraged_code)

# Ensure 'views_encouraged' is numeric
data['views_encouraged'] = pd.to_numeric(data['views_encouraged'], errors='coerce')

# Standardize the 'views_encouraged' and 'prevented_offensive' columns
scaler = StandardScaler()
data['views_encouraged_scaled'] = scaler.fit_transform(data[['views_encouraged']])
data['prevented_offensive_scaled'] = scaler.fit_transform(data[['prevented_offensive']].apply(lambda x: x.cat.codes))

# Calculate the difference and scale the result
data['campus_viewpoint_diversity'] = data['views_encouraged_scaled'] - data['prevented_offensive_scaled']

# Standardize the resulting difference
data['campus_viewpoint_diversity'] = scaler.fit_transform(data[['campus_viewpoint_diversity']])

# Discussion topic column names (Only interested in 'israelipalestinian_discuss' for regression)
columns_discuss = ['israelipalestinian_discuss']

# Controls
controls = ['politics_overall', 'politics_social', 'gender_recoded', 'extraversion', 'neuroticism', 'agreeableness',
            'conscientiousness', 'openness', 'typical_discuss']

# Dictionary to recode the political spectrum to be binary Left (0) vs Right (1)
politics_coding = {"Very right": 1,
                   "Right": 1,
                   "Lean right": 1,
                   "Lean left": 0,
                   "Left": 0,
                   "Very left": 0}

# Dictionary to recode the reluctance ratings
resp_coding = {"Very comfortable": 1,
               "Somewhat comfortable": 2,
               "Somewhat reluctant": 3,
               "Very reluctant": 4}

# Dictionary to recode gender
gender_coding = {"Male": 0,
                 "Other": 1,
                 "Female": 2}

# Recode the data for the political binary on economic, social, and overall political leanings
data['politics_economic'] = data['politics_economic'].replace(politics_coding)
data['politics_social'] = data['politics_social'].replace(politics_coding)
data['politics_overall'] = data['politics_overall'].replace(politics_coding)
data['gender_recoded'] = data['gender_recoded'].replace(gender_coding)

# Recode the reluctance ratings for each discussion column in prep for ordinal analyses
data[columns_discuss] = data[columns_discuss].replace(resp_coding)
data['typical_discuss'] = data['typical_discuss'].replace(resp_coding)

# Create 'campus_viewpoint_diversity' as specified
data['views_encouraged'] = pd.to_numeric(data['views_encouraged'], errors='coerce')
data['freeexpression_campu'] = pd.Categorical(data['freeexpression_campu'], categories=["Strongly disagree", "Somewhat disagree", "Somewhat agree", "Strongly agree"], ordered=True)
data['prevented_offensive'] = data['freeexpression_campu'].cat.codes
data['campus_viewpoint_diversity'] = StandardScaler().fit_transform(data[['views_encouraged']]) - StandardScaler().fit_transform(data[['prevented_offensive']])

# Add 'campus_viewpoint_diversity' to controls
controls.append('campus_viewpoint_diversity')

# Drop NaN from the columns we are interested in
columns_interest = ['israelipalestinian_discuss', 'politics_discuss', 'trans_discuss', 'abortion_discuss',
                    'religion_discuss', 'genderroles_discuss', 'sexualorientation_discuss', 'race_discuss',
                    'covid_discuss', 'freespeech_discuss', 'typical_discuss', 'politics_overall', 'gender_recoded',
                    'extraversion', 'neuroticism', 'agreeableness', 'conscientiousness', 'openness',
                    'campus_viewpoint_diversity']

for col in columns_interest:
  data.dropna(subset=[col], inplace=True)

# Standardize the data in order to get the standardized coefficient
scaler = StandardScaler()
data[controls] = scaler.fit_transform(data[controls])

# Function for the regressions
def fit_ordinal_model(data, predictors, outcome):
    model = OrderedModel(data[outcome], data[predictors], distr='logit')
    result = model.fit(method='bfgs', disp=False)
    print(f"Results for {outcome}:")
    print(result.summary())

    # Extract standardized coefficients (effect sizes)
    std_coefficients = result.params
    print("Standardized Coefficients (Effect Sizes):")
    print(std_coefficients)

    return result

# Run the regression only for 'israelipalestinian_discuss'
print('israelipalestinian_discuss')
results = fit_ordinal_model(data, controls, 'israelipalestinian_discuss')

# Categorize campus viewpoint diversity
data['viewpoint_category'] = pd.cut(
    data['campus_viewpoint_diversity'],
    bins=[-float('inf'), -1, 0, 1, float('inf')],
    labels=['Much Less Supportive', 'Less Supportive', 'More Supportive', 'Much More Supportive']
)

# Calculate the average reluctance score for 'israelipalestinian_discuss' by viewpoint category
average_reluctance_by_viewpoint = data.groupby('viewpoint_category')['israelipalestinian_discuss'].mean().reset_index()

# Plot the average reluctance score
plt.figure(figsize=(10, 6))
sns.barplot(x='viewpoint_category', y='israelipalestinian_discuss', data=average_reluctance_by_viewpoint, palette='viridis')
plt.xlabel('Campus Viewpoint Diversity', fontsize=12, fontweight='bold')
plt.ylabel('Overall Reluctance', fontsize=12, fontweight='bold')
plt.title('Average Reluctance Score for Israeli-Palestinian Discussions by Campus Viewpoint Diversity', fontsize=14, fontweight='bold')
plt.xticks(fontsize=10, fontweight='bold')
plt.yticks(fontsize=10, fontweight='bold')
plt.ylim(1, 4)
plt.tight_layout()
plt.show()

